ADCO_DESCRIPTION = "Adresse du compteur ADCO"
OPTARIF_DESCRIPTION = "Option tarifaire choisie"
ISOUSC_DESCRIPTION = "Intensité souscrite"
BASE_DESCRIPTION = "Index Base"
HCHC_DESCRIPTION = "Index Heures Creuses"
HCHP_DESCRIPTION = "Index Heures Pleines"
PTEC_DESCRIPTION = "Période Tarifaire en cours "
IINST_DESCRIPTION = "Intensité Instantanée"
IMAX_DESCRIPTION = "Intensité maximale appelée"
PAPP_DESCRIPTION = "Puissance apparente"
HHPHC_DESCRIPTION = "Horaire Heures Pleines Heures Creuses"
MOTDETAT_DESCRIPTION = "Mot d'état du compteur MOTDETAT"

UNIT_NONE = ""
UNIT_VA = "VA"
UNIT_WH = "Wh"
UNIT_A = "A"